#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
import os

figSave = 'off'
model = 'cryho_disclose_v1'

fileN = '2014_v3_at39_di51_id186_ai57'

Header = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse']

Unit =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1']

#======================================
#========== main
def main():
	#===== set path
	indir = '../%s/out_cal_obs'%(model)
	outdir = '../fig/%s'%(model)

	#===== read csv data
	data_all = rd_csv(indir)

	data_all[4,:] = data_all[4,:] - data_all[4,0]

	#=== plot figure
	make_subplt(outdir,data_all)

#======================================
#===== set path & read model data
def rd_csv(indir):

	if len(fileN) == 0:
		infile = '%s/ch_hourly.csv'%(indir)
	else:
		infile = '%s/ch_hourly_%s.csv'%(indir,fileN)
	print 'infile',infile

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		exit()
		data = -999,-999
		return data

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print data.shape

	for i in range(len(Header)):
		data[i,:] = np.array(df[Header[i]].values.tolist())
		if i == 16 or i == 17:
			data[i,:] = np.rad2deg(data[i,:])
		print "%s mean, min, max = %.02f, %.02f, %.02f"%(Header[i],np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))

	return data

#======================================
#===== plot figure
def make_subplt(outdir,d):

	#=== set plot param
	fig, axes = plt.subplots(6, 5, figsize=(20,15))
	plt.subplots_adjust(wspace=0.2,hspace=0.4) # adjust plot space

	print d.shape

	#=== plot
	v,h = 0,0
	for i in range(3,len(Header)):
		print v,h,Header[i]
		axes[v][h].plot(d[0,:],d[i,:],color='black',linestyle='-',lw=1.5)
		axes[v][h].set_title('%s %s'%(Header[i],Unit[i]),size=15)

		h = h + 1
		if h == 5:
			v = v + 1
			h = 0

	figN = 'cryho_ch_hourly_%s_subp'%fileN

	#===== save or show figure
	if figSave=='on':
		print'### save figure ###'
		if os.path.isdir('%s'%outdir) == False: os.makedirs('%s'%outdir)
		plt.savefig('%s/%s.png'%(outdir,figN),format='png',bbox_inches="tight")
	else:
		plt.show()
	print 'Out dir = %s'%outdir
	print 'Figure name = %s.png'%(figN)

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.0.0'
__date__ = 'Dec. 2022'

