#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
import os

figSave = 'off'
yaxis = 'value'	# fraction or value
yaxis2 = 'daily'	# daily or hourly

model = 'cryho_disclose_v1'

fileN_ctl = 'ch_hourly_2014_st_sr1'
fileN_st = 'ch_hourly_2014_st'

stList = ['at','sr','d0','phi','ai','ac','kisf','kidf']

Header = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse','src_diffuse_t']

Unit =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1','W/m2']

srList = ['sr2','sr3']

#======================================
#========== main
def main():
	#===== set path
	indir_ctl = '../%s/out_stest'%model
	indir_st = '../%s/out_stest'%model
	outdir = '../fig/%s'%model

	#===== read csv data (control)
	d_ctl = rd_csv(indir_ctl,fileN_ctl,Header,'','ctl')

	#===== read csv data (sensitivity test)
	sr_all = np.zeros((len(srList),d_ctl.shape[0],d_ctl.shape[1]))
	for i in range(len(srList)):
		sr_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,srList[i],'sr')

	#=== plot figure
	make_plt_sr(outdir,d_ctl,sr_all,yaxis)

#======================================
#===== set path & read model data
def rd_csv(indir,fileN,Header,st,opt):

	if opt == 'ctl':
		infile = '%s/%s.csv'%(indir,fileN)
	else:
		print st
		infile = '%s/%s_%s.csv'%(indir,fileN,st)

	print 'infile',infile

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		exit()

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print opt,data.shape

	for i in range(len(Header)):
		data[i,:] = np.array(df[Header[i]].values.tolist())
		print "%s mean, min, max = %.02f, %.02f, %.02f"%(Header[i],np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))
		
	return data

#======================================
#===== plot figure
def make_plt_sr(outdir,d_ctl,d_sr,yaxis):

	#=== set plot param
	fig, axes = plt.subplots(1, 1, figsize=(12,8))
	plt.subplots_adjust(wspace=0.05,hspace=0.15) # adjust plot space

	colorList = ['blue','aqua','lime','orange','red']
	Label = ['Direct','Diffuse']
	unit = ''
	ctl_v = 'Direct and Diffuse'
	doy = d_ctl[0,:]
	ang_c = np.rad2deg(d_ctl[16,:])
	ang_z = np.rad2deg(d_ctl[17,:])

	#=== control
	sr_dir1 = d_ctl[25,:]
	sr_dif1 = d_ctl[26,:]
	src_dir1 = d_ctl[27,:]
	src_dif_t1 = d_ctl[-1,:]
	src_dif_d1 = d_ctl[28,:] - src_dif_t1
	src_all1 = src_dir1 + src_dif_t1 + src_dif_d1
	src_all_r1 = src_all1 / (sr_dir1 + sr_dif1) * 100
	src_dir_r1 = src_dir1 / sr_dir1 * 100
	src_dif_r1 = (src_dif_t1 + src_dif_d1) / sr_dif1 * 100

	src_dir_t1 = np.zeros((len(doy)))
	src_dir_d1 = np.zeros((len(doy)))
	for i in range(len(doy)):
		if ang_z[i] > ang_c[i]:
			src_dir_t1[i] = src_dir1[i]
			src_dir_d1[i] = 0
		else:
			src_dir_t1[i] = 0
			src_dir_d1[i] = src_dir1[i]

	src_t1 = src_dir_t1 + src_dif_t1
	src_d1 = src_dir_d1 + src_dif_d1
	src_t_r1 = src_t1 / (sr_dir1 + sr_dif1) * 100
	src_d_r1 = src_d1 / (sr_dir1 + sr_dif1) * 100

	#=== direct
	sr_dir2 = d_sr[0,25,:]
	src_dir2 = d_sr[0,27,:]
	src_dir_r2 = src_dir2 / sr_dir2 * 100

	#=== diffuse
	sr_dif3 = d_sr[1,26,:]
	src_dif_t3 = d_sr[1,-1,:]
	src_dif_d3 = d_sr[1,28,:] - src_dif_t3
	src_dif_d_r3 = src_dif_d3 / sr_dif3 * 100
	src_dif_t_r3 = src_dif_t3 / sr_dif3 * 100
	src_dif_r3 = src_dif_d_r3 + src_dif_t_r3
	src_dif3 = src_dif_d3 + src_dif_t3

	src_all_r1_daily = np.zeros((30))
	src_t_r1_daily = np.zeros((30))
	src_d_r1_daily = np.zeros((30))
	src_dir_r1_daily = np.zeros((30))
	src_dif_r1_daily = np.zeros((30))
	src_dir_r2_daily = np.zeros((30))
	src_dif_r3_daily = np.zeros((30))

	src_all1_daily = np.zeros((30))
	src_t1_daily = np.zeros((30))
	src_d1_daily = np.zeros((30))
	src_dir2_daily = np.zeros((30))
	src_dif3_daily = np.zeros((30))

	for i in range(len(src_all_r1)/24):
		src_all_r1_daily[i] = np.nanmean(src_all_r1[i*24:i*24+24])
		src_t_r1_daily[i] = np.nanmean(src_t_r1[i*24:i*24+24])
		src_d_r1_daily[i] = np.nanmean(src_d_r1[i*24:i*24+24])
		src_dir_r1_daily[i] = np.nanmean(src_dir_r1[i*24:i*24+24])
		src_dif_r1_daily[i] = np.nanmean(src_dif_r1[i*24:i*24+24])
		src_dir_r2_daily[i] = np.nanmean(src_dir_r2[i*24:i*24+24])
		src_dif_r3_daily[i] = np.nanmean(src_dif_r3[i*24:i*24+24])
		src_all1_daily[i] = np.nanmean(src_all1[i*24:i*24+24])
		src_t1_daily[i] = np.nanmean(src_t1[i*24:i*24+24])
		src_d1_daily[i] = np.nanmean(src_d1[i*24:i*24+24])
		src_dir2_daily[i] = np.nanmean(src_dir2[i*24:i*24+24])
		src_dif3_daily[i] = np.nanmean(src_dif3[i*24:i*24+24])
 
	#=== plot (daily)
	if yaxis == 'fraction' and yaxis2 == 'daily':
		axes.plot(doy[::24],src_all_r1_daily,color='black',linestyle='-',ls=1.5,
				#label='Control\n($\mathregular{R_{Sdc}}$+$\mathregular{R_{Sfc}}$+$\mathregular{R_{Stdc}}$+$\mathregular{R_{Stfc}}$)')
				label='All components in Ctl-exp')
		axes.plot(doy[::24],src_d_r1_daily,color='grey',linestyle='-',ls=1.5,
				#label='Control ($\mathregular{R_{Sdc}}$+$\mathregular{R_{Sfc}}$)')
				label='Components from hole mouth in Ctl-exp')
		axes.plot(doy[::24],src_t_r1_daily,color='grey',linestyle='--',ls=1.5,
				#label='Control ($\mathregular{R_{Stdc}}$+$\mathregular{R_{Stfc}}$)')
				label='Ice-transmitted components in Ctl-exp')
		axes.plot(doy[::24],src_dir_r2_daily,color='red',linestyle='-',ls=1.5,
				#label='Direct ($\mathregular{R_{Sdc}}$+$\mathregular{R_{Stdc}}$)')
				label='Direct components in $\mathregular{R_{s}}$-exp')
		axes.plot(doy[::24],src_dif_r3_daily,color='blue',linestyle='-',ls=1.5,
				#label='Diffuse ($\mathregular{R_{Sfc}}$+$\mathregular{R_{Stfc}}$)')
				label='Diffuse components in $\mathregular{R_{s}}$-exp')
		figN = 'fig10_frc_daily'
	elif yaxis == 'value' and yaxis2 == 'daily':
		axes.plot(doy[::24],src_all1_daily,color='black',linestyle='-',ls=1.5,
				label='All components in Ctl-exp')
		axes.plot(doy[::24],src_d1_daily,color='grey',linestyle='-',ls=1.5,
				label='Components from hole mouth in Ctl-exp')
		axes.plot(doy[::24],src_t1_daily,color='grey',linestyle='--',ls=1.5,
				label='Ice-transmitted components in Ctl-exp')
		axes.plot(doy[::24],src_dir2_daily,color='red',linestyle='-',ls=1.5,
				label='Direct components in $\mathregular{R_{s}}$-exp')
		axes.plot(doy[::24],src_dif3_daily,color='blue',linestyle='-',ls=1.5,
				label='Diffuse components in $\mathregular{R_{s}}$-exp')
		figN = 'fig10_val_daily'


	#=== plot (hourly)
	if yaxis == 'fraction' and yaxis2 == 'hourly':
		axes.plot(doy,src_all_r1,color='black',linestyle='-',ls=1.5,
				label='All components in Ctl-exp')
		axes.plot(doy,src_d_r1,color='grey',linestyle='-',ls=1.5,
				label='Components from hole mouth in Ctl-exp')
		axes.plot(doy,src_t_r1,color='grey',linestyle='--',ls=1.5,
				label='Ice-transmitted components in Ctl-exp')
		axes.plot(doy,src_dir_r2,color='red',linestyle='-',ls=1.5,
				label='Direct components in $\mathregular{R_{s}}$-exp')
		axes.plot(doy,src_dif_r3,color='blue',linestyle='-',ls=1.5,
				label='Diffuse components in $\mathregular{R_{s}}$-exp')
		figN = 'fig10_frc_hourly'
	elif yaxis == 'value' and yaxis2 == 'hourly':
		axes.plot(doy,src_all1,color='black',linestyle='-',ls=1.5,
				label='All components in Ctl-exp')
		axes.plot(doy,src_d1,color='grey',linestyle='-',ls=1.5,
				label='Components from hole mouth in Ctl-exp')
		axes.plot(doy,src_t1,color='grey',linestyle='--',ls=1.5,
				label='Ice-transmitted components in Ctl-exp')
		axes.plot(doy,src_dir2,color='red',linestyle='-',ls=1.5,
				label='Direct components in $\mathregular{R_{s}}$-exp')
		axes.plot(doy,src_dif3,color='blue',linestyle='-',ls=1.5,
				label='Diffuse components in $\mathregular{R_{s}}$-exp')
		figN = 'fig10_val_hourly'

	#=== set label axis
	axes.set_xlim(xmin=182)
	axes.set_xlim(xmax=218)

	if yaxis == 'fraction':
		axes.set_ylim(ymin=0)
		axes.set_ylim(ymax=100)
		axes.set_ylabel("Reaching fraction to cryoconite hole bottom (%)",fontsize=24)
	elif yaxis == 'value':
		axes.set_ylim(ymin=0)
		axes.set_ylim(ymax=300)
		axes.set_ylabel("Shortwave radiation reaching\ncryoconite hole bottom (W $\mathregular{m^{-2}}$)",fontsize=24)

	axes.tick_params(axis='y',labelsize=20)
	axes.tick_params(axis='x',labelsize=20)
	axes.set_xlabel("Day of year",fontsize=24)
	#axes.legend(loc='lower right',fontsize=19)
	axes.legend(loc='upper left',fontsize=19)

	#===== save or show figure

	if figSave=='on':
		print'### save figure ###'
		if os.path.isdir('%s'%outdir) == False: os.makedirs('%s'%outDir)
		plt.savefig('%s/%s.png'%(outdir,figN),format='png',bbox_inches="tight")
	else:
		plt.show()
	print 'Out dir = %s'%outdir
	print 'Figure name = %s.png'%(figN)

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'May 2023'
