#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
import os

figSave = 'off'
model = 'cryho_disclose_v1'

Year = 2014

fileN_example = 'ch_hourly_%d_st_Rscphi10_LT01'%Year
fileN_st = 'ch_hourly_%d_st'%Year

stList = ['d','phi']

Header = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse','src_diffuse_t']

Unit =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1','W/m2']

dList = ['Rscd10','Rscd50','Rscd100','Rscd150','Rscd200','Rscd250','Rscd300','Rscd350']
phiList = ['Rscphi10','Rscphi30','Rscphi50','Rscphi70','Rscphi90','Rscphi110','Rscphi130','Rscphi150']

#======================================
#========== main
def main():
	#===== set path
	indir_example = '../%s/out_stest_Rscexp'%model
	indir_st = '../%s/out_stest_Rscexp'%model
	outdir = '../fig/%s'%model

	#===== read csv data (get array size)
	d_example = rd_csv(indir_example,fileN_example,Header,'','ctl','')

	d_all_LT13 = np.zeros((len(dList),d_example.shape[0],d_example.shape[1]))
	phi_all_LT13 = np.zeros((len(phiList),d_example.shape[0],d_example.shape[1]))
	d_all_LT01 = np.zeros((len(dList),d_example.shape[0],d_example.shape[1]))
	phi_all_LT01 = np.zeros((len(phiList),d_example.shape[0],d_example.shape[1]))

	#===== read csv data (sensitivity test)
	for i in range(len(dList)):
		d_all_LT13[i,:,:] = rd_csv(indir_st,fileN_st,Header,dList[i],'d','LT13')
		d_all_LT01[i,:,:] = rd_csv(indir_st,fileN_st,Header,dList[i],'d','LT01')
	for i in range(len(phiList)):
		phi_all_LT13[i,:,:] = rd_csv(indir_st,fileN_st,Header,phiList[i],'phi','LT13')
		phi_all_LT01[i,:,:] = rd_csv(indir_st,fileN_st,Header,phiList[i],'phi','LT01')

	#=== plot figure
	make_subplt_Rscexp(outdir,d_all_LT13,phi_all_LT13,d_all_LT01,phi_all_LT01)

#======================================
#===== set path & read model data
def rd_csv(indir,fileN,Header,st,opt,lt):

	if opt == 'ctl':
		infile = '%s/%s.csv'%(indir,fileN)
	else:
		print st
		infile = '%s/%s_%s_%s.csv'%(indir,fileN,st,lt)

	print 'infile',infile

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		exit()

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print opt,data.shape

	for i in range(len(Header)):
		data[i,:] = np.array(df[Header[i]].values.tolist())
		#print "%s mean, min, max = %.02f, %.02f, %.02f"%(Header[i],np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))
		
	return data

#======================================
#===== plot figure
def make_subplt_Rscexp(outdir,d_d_LT13,d_phi_LT13,d_d_LT01,d_phi_LT01):

	#=== set plot param
	fig, axes = plt.subplots(2, 2, figsize=(20,16))
	plt.subplots_adjust(wspace=0.05,hspace=0.15) # adjust plot space

	d_list = [d_d_LT13,d_phi_LT13,d_d_LT01,d_phi_LT01]

	#=== plot (temp)
	i,j = 0,0
	t1 = 492 # 492=doy172LT13:00
	t2 = 480 # 480=doy172LT01:00
	for n in range(4):
		if i == 0:
			t = t1
		elif i == 1:
			t = t2
		st = stList[j]
		print st
		print d_list[n].shape
		print "mon,day,hour",d_list[n][0,1,t],d_list[n][0,2,t],d_list[n][0,3,t]
		ang_c = np.rad2deg(d_list[n][:,16,t])
		ang_z = np.rad2deg(d_list[n][:,17,t])
		cd = d_list[n][:,23,t]
		sr_dir = d_list[n][:,25,t]
		sr_dif = d_list[n][:,26,t]
		src_dir = d_list[n][:,27,t]
		src_dif_t = d_list[n][:,-1,t]
		src_dif_d = d_list[n][:,28,t] - src_dif_t

		src_dir_r = src_dir / sr_dir * 100
		src_dif_d_r = src_dif_d / sr_dif * 100
		src_dif_t_r = src_dif_t / sr_dif * 100

		bar_st(ang_c,ang_z,src_dir_r,src_dif_d_r,src_dif_t_r,st,axes[i][j])

		j = j + 1
		if j == 2:
		 	i = i + 1
			j = 0

	#=== set label axis
	fig_list = ['a)','b)','c)','d)']
	ang_z1 = np.rad2deg(d_list[0][0,17,t1])
	ang_z2 = np.rad2deg(d_list[0][0,17,t2])
	print ang_z1, ang_z2
	thetaz1 = u'%d\N{DEGREE SIGN}'%ang_z1
	thetaz2 = u'%d\N{DEGREE SIGN}'%ang_z2

	i,j = 0,0
	for n in range(4):

		axes[i][j].set_ylim(ymin=0)
		axes[i][j].set_ylim(ymax=100)

		axes[i][j].tick_params(axis='y',labelsize=20)
		axes[i][j].tick_params(axis='x',labelsize=20)

		#axes[i][j].set_facecolor('lightgrey')

		if i == 1 and j == 0:
			axes[i][j].set_xlabel("Cryoconite hole depth $D$ (m)",fontsize=30)
		elif i == 1 and j == 1:
			axes[i][j].set_xlabel("Cryoconite hole diameter $\Phi$ (m)",fontsize=30)
		else:
			axes[i][j].xaxis.set_ticklabels([])
	
		if j == 1:
			axes[i][j].yaxis.set_ticklabels([])
		else:
			axes[i][j].set_ylabel("Reaching fraction to\ncryoconite hole bottom (%)",fontsize=30)

		if i == 0 and j == 0:
			axes[i][j].text(0.17,0.92,fig_list[n],transform=axes[i][j].transAxes,size=26)
			axes[i][j].text(0.17,0.83,'$\Phi$ = 0.05 m',transform=axes[i][j].transAxes,size=24)
			axes[i][j].text(0.17,0.74,r'$\theta_{z}$ = %s'%thetaz1,transform=axes[i][j].transAxes,size=24)
		elif i == 0 and j == 1:
			axes[i][j].text(0.030,0.92,fig_list[n],transform=axes[i][j].transAxes,size=26)
			axes[i][j].text(0.015,0.83,'$D$ = 0.01 m',transform=axes[i][j].transAxes,size=24)
			axes[i][j].text(0.015,0.74,r'$\theta_{z}$ = %s'%thetaz1,transform=axes[i][j].transAxes,size=24)
		elif i == 1 and j == 0:
			axes[i][j].text(0.17,0.92,fig_list[n],transform=axes[i][j].transAxes,size=26)
			axes[i][j].text(0.17,0.83,'$\Phi$ = 0.05 m',transform=axes[i][j].transAxes,size=24)
			axes[i][j].text(0.17,0.74,r'$\theta_{z}$ = %s'%thetaz2,transform=axes[i][j].transAxes,size=24)
		elif i == 1 and j == 1:
			axes[i][j].text(0.030,0.92,fig_list[n],transform=axes[i][j].transAxes,size=26)
			axes[i][j].text(0.015,0.83,'$D$ = 0.01 m',transform=axes[i][j].transAxes,size=24)
			axes[i][j].text(0.015,0.74,r'$\theta_{z}$ = %s'%thetaz2,transform=axes[i][j].transAxes,size=24)


		if i == 0 and j == 0:
			axes[i][j].legend(loc='upper right',fontsize=26)

		j = j + 1
		if j == 2:
			i = i + 1
			j = 0

	#===== save or show figure
	figN = 'fig11'

	if figSave=='on':
		print'### save figure ###'
		if os.path.isdir('%s'%outdir) == False: os.makedirs('%s'%outDir)
		plt.savefig('%s/%s.png'%(outdir,figN),format='png',bbox_inches="tight")
	else:
		plt.show()
	print 'Out dir = %s'%outdir
	print 'Figure name = %s.png'%(figN)

#======================================
#===== plot figure
def bar_st(ang_c,ang_z,src_dir_r,src_dif_d_r,src_dif_t_r,st,axes):
	#global colorList,colorList_at

	#clist = ['blue','red','orange']
	clist = ['tomato','dodgerblue','orange']

	yunit = 'W/m2'

	if st == "d":
		Label = ['0.01','0.05','0.10','0.15','0.20','0.25','0.30','0.35']
		xunit = 'm'
	elif st == "phi":
		Label = ['0.01','0.03','0.05','0.07','0.09','0.11','0.13','0.15']
		xunit = 'm'

	print 'exp %s'%st

	for i in range(len(Label)):

		if ang_c[i] > ang_z[i]:
			if i == 0:
				axes.bar(x=Label[i],height=src_dir_r[i],bottom=0,color=clist[0],align="edge",alpha=1,width=-0.3,ec='k',label='$\mathregular{R_{Sdc}}$')
			else:
				axes.bar(x=Label[i],height=src_dir_r[i],bottom=0,color=clist[0],align="edge",alpha=1,width=-0.3,ec='k')
		else:
			if i == 1:
				axes.bar(x=Label[i],height=src_dir_r[i],bottom=0,color=clist[0],align="edge",alpha=0.5,width=-0.3,ec='k',label='$\mathregular{R_{Stdc}}$')
			else:
				axes.bar(x=Label[i],height=src_dir_r[i],bottom=0,color=clist[0],align="edge",alpha=0.5,width=-0.3,ec='k')
		#= bar
		if i == 0:
			axes.bar(x=Label[i],height=src_dif_d_r[i],bottom=0,color=clist[1],align="edge",alpha=1,width=0.3,ec='k',label='$\mathregular{R_{Sfc}}$')
			bottom1 = src_dif_d_r[i]
			axes.bar(x=Label[i],height=src_dif_t_r[i],bottom=bottom1,color=clist[1],align="edge",alpha=0.5,width=0.3,ec='k')
		elif i == 1:
			axes.bar(x=Label[i],height=src_dif_d_r[i],bottom=0,color=clist[1],align="edge",alpha=1,width=0.3,ec='k')
			bottom1 = src_dif_d_r[i]
			axes.bar(x=Label[i],height=src_dif_t_r[i],bottom=bottom1,color=clist[1],align="edge",alpha=0.5,width=0.3,ec='k',label='$\mathregular{R_{Stfc}}$')
		else:
			axes.bar(x=Label[i],height=src_dif_d_r[i],bottom=0,color=clist[1],align="edge",alpha=1,width=0.3,ec='k')
			bottom1 = src_dif_d_r[i]
			axes.bar(x=Label[i],height=src_dif_t_r[i],bottom=bottom1,color=clist[1],align="edge",alpha=0.5,width=0.3,ec='k')

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'May 2023'

