#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
import os
import scipy.stats

figSave = 'off'
model = 'cryho_disclose_v1'

Year = 2014

fileN_met = 'SIGMA-B_LT20140701-_Lv1_3'
fileN_mod = 'ch_hourly_%d_v3_at39_di51_id186_ai57'%Year

Header_met = ['year','month','day','hour','doy','temp','ws','rh',
	'swdown','swup','lwdown','lwup','sd_obs','press']

Unit_met =   [''    ,''     ,''   ,''    ,''   ,'deg'   ,'m/s'  ,'-'   ,
	'W/m2'  ,'W/m2','W/m2'  ,'W/m2','m'     ,'hPa']

Header_mod = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse']

Unit_mod =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1']

#======================================
#========== main
def main():
	#===== set path
	indir_met = '../data/met_lv3'
	indir_mod = '../%s/out_cal_obs'%model
	outdir = '../fig/%s'%model

	#===== read csv data
	data_met = rd_csv(indir_met,fileN_met,Header_met,'met')
	data_met[np.where(data_met<-99) ] = np.nan

	data_mod = rd_csv(indir_mod,fileN_mod,Header_mod,'mod')

	#===== cal shallow (negative) or deppen (positive) phase (CH depth)
	ch_change = data_mod[23,:] - np.roll(data_mod[23,:],1)
	ch_change[0] = 0	# init val

	#=== plot figure
	splot_subp_chVSmet(outdir,ch_change,data_mod,data_met)

#======================================
#===== set path & read model data
def rd_csv(indir,fileN,Header,opt):

	infile = '%s/%s.csv'%(indir,fileN)
	print 'infile',infile

	if os.path.exists(infile) and opt == 'met':
		df = pd.read_csv(infile,names=Header,encoding='UTF-8')
	elif os.path.exists(infile) and opt == 'mod':
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		exit()

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print data.shape

	for i in range(len(Header)):
		print Header[i]
		data[i,:] = np.array(df[Header[i]].values.tolist())
		print "mean, min, max = %.02f, %.02f, %.02f"%(np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))

	#=== modify date		
	if opt == 'met' and Year == 2014:
		data = data[:,24*4:24*34]
	elif opt == 'met' and Year == 2017:
		#data = data[:,24*7:24*33]
		data = data[:,24*15:24*33]
		print data.shape

	return data

#======================================
#===== plot figure
def splot_subp_chVSmet(outdir,ch_change,d_mod,d_met):

	#=== set plot param
	fig, axes = plt.subplots(3, 5, figsize=(15,15))
	plt.subplots_adjust(wspace=0.15,hspace=0.35) # adjust plot space

	#u_deg = '${}^\circ\mathrm{C}$'
	u_deg = 'K'
	u_ms = 'm $\mathregular{s^{-1}}$'
	u_wm2 = 'W $\mathregular{m^{-2}}$'

	Unit_lab1 = [u_deg,u_wm2,u_wm2,u_ms,'-']
	Unit_lab2 = [u_wm2,u_wm2,u_wm2,u_wm2,u_wm2]
	Unit_lab3 = [u_wm2,u_wm2,u_wm2,u_wm2,u_wm2]

	Header_idx1 = ['temp','swdown','lwdown','ws','rh']
	Header_idx2 = ['melt_ice','sr_direct','sr_diffuse','sh_ice','lh_ice']
	Header_idx3 = ['melt_cryo','src_direct','src_diffuse','sh_cryo','lh_cryo']

	Header_lab1 = ['$T_{a}$','$R_{S}$','$R_{L}$','$U$','$h_{r}$']
	Header_lab2 = ['$Q_{i}$','$R_{Sd}$','$R_{Sf}$','$H_{Si}$','$H_{Li}$']
	Header_lab3 = ['$Q_{c}$','$R_{Sdc}+R_{Stdc}$\n','$R_{Sfc}+R_{Stfc}$\n','$H_{Sc}$','$H_{Lc}$']

	txtList = ['a','b','c','d','e','f','g','h','i','j','k','l','m']

	#=== plot
	v,h = 0,0
	for i in range(3*5):

		#= plt var
		if v == 0:
			print Header_idx1[h]
			var_idx = Header_met.index(Header_idx1[h])
			x_val_shl = d_met[var_idx,:][np.where(ch_change<0)]
			x_val_dep = d_met[var_idx,:][np.where(ch_change>0)]
			if h == 0:
				x_val_shl += 3.92 
				x_val_dep += 3.92 

		elif v == 1:
			print Header_idx2[h]
			var_idx = Header_mod.index(Header_idx2[h])
			x_val_shl = d_mod[var_idx,:][np.where(ch_change<0)]
			x_val_dep = d_mod[var_idx,:][np.where(ch_change>0)]

		elif v == 2:
			print Header_idx3[h]
			var_idx = Header_mod.index(Header_idx3[h])
			x_val_shl = d_mod[var_idx,:][np.where(ch_change<0)]
			x_val_dep = d_mod[var_idx,:][np.where(ch_change>0)]

		y_val_shl = ch_change[np.where(ch_change<0)] * 0.001 # mm -> m
		y_val_dep = ch_change[np.where(ch_change>0)] * 0.001

		if i < 13:	# remove HS HL at CH bottom
			axes[v][h].scatter(x_val_dep,y_val_dep,c='blue',s=15,alpha=0.3)
			axes[v][h].scatter(x_val_shl,y_val_shl,c='red',s=15,alpha=0.3)
			axes[v][h].text(0.04,0.88,'%s)'%txtList[i],transform=axes[v][h].transAxes,size=20)
			rd,pd = scipy.stats.pearsonr(x_val_dep,y_val_dep)
			rs,ps = scipy.stats.pearsonr(x_val_shl,y_val_shl)
			#'a','b','c','d','e','f','g','h','i','j','k','l','m'
			if txtList[i] in ['a','c','d','e','f']:
				axes[v][h].text(0.04,0.15,'$r_{d}$=%.02f'%rd,transform=axes[v][h].transAxes,size=18)
				axes[v][h].text(0.04,0.05,'$r_{s}$=%.02f'%rs,transform=axes[v][h].transAxes,size=18)
			elif txtList[i] in ['b','g','h','k','l','m']:
				axes[v][h].text(0.44,0.15,'$r_{d}$=%.02f'%rd,transform=axes[v][h].transAxes,size=18)
				axes[v][h].text(0.44,0.05,'$r_{s}$=%.02f'%rs,transform=axes[v][h].transAxes,size=18)
			elif txtList[i] in ['i','j']:
				axes[v][h].text(0.44,0.85,'$r_{d}$=%.02f'%rd,transform=axes[v][h].transAxes,size=18)
				axes[v][h].text(0.44,0.75,'$r_{s}$=%.02f'%rs,transform=axes[v][h].transAxes,size=18)
			
		#= set label
		if h == 0:
			axes[v][h].set_ylabel("$dD/dt$ (m $\mathregular{h^{-1}})$",fontsize=20)
		else:
			axes[v][h].yaxis.set_ticklabels([])

		axes[v][h].tick_params(axis='y',labelsize=17)
		axes[v][h].tick_params(axis='x',labelsize=17)
		axes[v][h].set_ylim(ymin=-0.004)
		axes[v][h].set_ylim(ymax=0.0025)

		if v == 0:
			axes[v][h].set_xlabel('%s (%s)'%(Header_lab1[h],Unit_lab1[h]),fontsize=20)
		elif v == 1:
			axes[v][h].set_xlabel('%s (%s)'%(Header_lab2[h],Unit_lab2[h]),fontsize=20)
		elif v == 2:
			axes[v][h].set_xlabel('%s (%s)'%(Header_lab3[h],Unit_lab3[h]),fontsize=20)

		if i < 13:
			axes[v][h].axhline(y=0,color='black',ls='--')

		if i > 12:
			axes[v][h].axis("off")
		
		h = h + 1
		if h == 5:
			v = v + 1
			h = 0

	#== legend
	plt.scatter([],[],c='blue',s=30,label='Deepening phase')
	plt.scatter([],[],c='red',s=30,label='Shallowing phase')
	plt.legend(loc='lower right',fontsize=22, bbox_to_anchor=(0.9,0.25))

	#===== save or show figure
	figN = 'fig12'

	if figSave=='on':
		print'### save figure ###'
		if os.path.isdir('%s'%outdir) == False: os.makedirs('%s'%outDir)
		plt.savefig('%s/%s.png'%(outdir,figN),format='png',bbox_inches="tight")
	else:
		plt.show()
	print 'Out dir = %s'%outdir
	print 'Figure name = %s.png'%(figN)

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.1.0'
__date__ = 'May 2023'

