#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
import pandas as pd
import sys
import os

figSave = 'off'
model = 'cryho_disclose_v1'

Year = 2014

fileN_met = 'SIGMA-B_LT20140701-_Lv1_3'
fileN_mod = 'ch_hourly_%d_v3_at39_di51_id186_ai57'%Year

Header_met = ['year','month','day','hour','doy','temp','ws','rh',
	'swdown','swup','lwdown','lwup','sd_obs','press']

Unit_met =   [''    ,''     ,''   ,''    ,''   ,'deg'   ,'m/s'  ,'-'   ,
	'W/m2'  ,'W/m2','W/m2'  ,'W/m2','m'     ,'hPa']

Header_mod = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse']

Unit_mod =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1']

#======================================
#========== main
def main():
	#===== set path
	indir_met = '../data/met_lv3'
	indir_mod = '../%s/out_cal_obs'%model
	outdir = '../fig/%s'%model

	#===== read csv data
	data_met = rd_csv(indir_met,fileN_met,Header_met,'met')
	data_met[np.where(data_met<-99) ] = np.nan

	data_mod = rd_csv(indir_mod,fileN_mod,Header_mod,'mod')

	#===== cal shallow (negative) or deppen (positive) phase (CH depth)
	ch_change = data_mod[23,:] - np.roll(data_mod[23,:],1)
	ch_change[0] = 0	# init val

	#=== plot figure
	splot_subp_chVSmet(outdir,ch_change,data_mod,data_met)

#======================================
#===== set path & read model data
def rd_csv(indir,fileN,Header,opt):

	infile = '%s/%s.csv'%(indir,fileN)
	print 'infile',infile

	if os.path.exists(infile) and opt == 'met':
		df = pd.read_csv(infile,names=Header,encoding='UTF-8')
	elif os.path.exists(infile) and opt == 'mod':
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		exit()

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print data.shape

	for i in range(len(Header)):
		print Header[i]
		data[i,:] = np.array(df[Header[i]].values.tolist())
		print "mean, min, max = %.02f, %.02f, %.02f"%(np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))

	#=== modify date		
	if opt == 'met' and Year == 2014:
		data = data[:,24*4:24*34]
	elif opt == 'met' and Year == 2017:
		#data = data[:,24*7:24*33]
		data = data[:,24*15:24*33]
		print data.shape

	return data

#======================================
#===== plot figure
def splot_subp_chVSmet(outdir,ch_change,d_mod,d_met):

	#=== set plot param
	fig, axes = plt.subplots(1, 2, figsize=(19,8))
	plt.subplots_adjust(wspace=0.3,hspace=0.15) # adjust plot space

	u_deg = '${}^\circ\mathrm{C}$'
	u_ms = 'm $\mathregular{s^{-1}}$'
	u_wm2 = 'W $\mathregular{m^{-2}}$'

	Unit_lab1 = [u_deg,u_wm2,u_wm2,u_ms,'-']
	Unit_lab2 = [u_wm2,u_wm2,u_wm2,u_wm2,u_wm2]
	Unit_lab3 = [u_wm2,u_wm2,u_wm2,u_wm2,u_wm2]

	Header_idx1 = ['temp','swdown','lwdown','ws','rh']
	Header_idx2 = ['melt_ice','sr_direct','sr_diffuse','sh_ice','lh_ice']
	Header_idx3 = ['melt_cryo','src_direct','src_diffuse','sh_cryo','lh_cryo']

	Header_lab1 = ['$T_{a}$','$R_{S}$','$R_{L}$','$U$','$h_{r}$']
	Header_lab2 = ['$Q_{i}$','$R_{Sd}$','$R_{Sf}$','$H_{Si}$','$H_{Li}$']
	Header_lab3 = ['$Q_{c}$','$R_{Sdc}+R_{Stdc}$','$R_{Sfc}+R_{Stfc}$','$H_{Sc}$','$H_{Lc}$']

	yminList = [-10,0,0,40]
	ymaxList = [10,700,15,100]
	txtList = ['a','b','c','d','e','f','g','h','i','j','k','l','m']

	#= plt var
	print Header_idx1[3]
	var_idx1 = Header_met.index(Header_idx1[3])
	x_val_shl1 = d_met[var_idx1,:][np.where(ch_change<0)]
	x_val_dep1 = d_met[var_idx1,:][np.where(ch_change>0)]

	print Header_idx3[2]
	var_idx3 = Header_mod.index(Header_idx3[2])
	x_val_shl3 = d_mod[var_idx3,:][np.where(ch_change<0)]
	x_val_dep3 = d_mod[var_idx3,:][np.where(ch_change>0)]

	y_val_shl = ch_change[np.where(ch_change<0)]
	y_val_dep = ch_change[np.where(ch_change>0)]

	cs= axes[0].scatter(x_val_shl3,y_val_shl*0.001,c=x_val_shl1,cmap='jet',s=35)
	axes[0].text(0.84,0.88,'%s)'%txtList[0],transform=axes[0].transAxes,size=30)

	axes[1].scatter(x_val_dep3,y_val_dep*0.001,c=x_val_dep1,cmap='jet',s=35)
	axes[1].text(0.08,0.88,'%s)'%txtList[1],transform=axes[1].transAxes,size=30)

	#= set label
	axes[0].set_ylabel("Shallowing rate $dD/dt$ (m $\mathregular{h^{-1}})$",fontsize=24)
	axes[1].set_ylabel("Deepening rate $dD/dt$ (m $\mathregular{h^{-1}})$",fontsize=24)

	#===== Cordinate Colorbar
	divider = make_axes_locatable(axes[0])
	cbar = plt.colorbar(cs, orientation='horizontal',cax=fig.add_axes([0.14,-0.06,0.75,0.030]))  #[x,y,l,w]
	cbar.ax.tick_params(labelsize=20)
	cbar.set_label('Wind speed $U$ (%s)'%Unit_lab1[3],fontsize=24)

	axes[0].tick_params(axis='y',labelsize=18)
	axes[0].tick_params(axis='x',labelsize=18)
	#axes[0].set_xlabel('Diffuse component of shortwave radiation reaching to the cryoconite hole bottom\n%s (%s)'%(Header_lab3[2],Unit_lab3[2]),fontsize=24)
	axes[0].set_xlabel('%s (%s)'%(Header_lab3[2],Unit_lab3[2]),fontsize=24)
	axes[0].set_ylim(ymin=-0.0035)
	axes[0].set_ylim(ymax=0)
	axes[0].set_xlim(xmin=0)
	axes[0].set_xlim(xmax=500)

	axes[1].tick_params(axis='y',labelsize=18)
	axes[1].tick_params(axis='x',labelsize=18)
	axes[1].set_xlabel('%s (%s)'%(Header_lab3[2],Unit_lab3[2]),fontsize=24)
	axes[1].set_ylim(ymin=0)
	axes[1].set_ylim(ymax=0.0025)
	axes[1].set_xlim(xmin=0)
	axes[1].set_xlim(xmax=500)

	#===== save or show figure
	figN = 'fig13'

	if figSave=='on':
		print'### save figure ###'
		if os.path.isdir('%s'%outdir) == False: os.makedirs('%s'%outDir)
		plt.savefig('%s/%s.png'%(outdir,figN),format='png',bbox_inches="tight")
	else:
		plt.show()
	print 'Out dir = %s'%outdir
	print 'Figure name = %s.png'%(figN)

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.1.0'
__date__ = 'May 2023'

