#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
import os

figSave = 'off'

model = 'cryho_disclose_v1'

Header_met = ['year','month','day','hour','doy','atemp','ws','rh',
	'swdown','swup','lwdown','lwup','sd_obs','press']

Unit_met =   [''    ,''     ,''   ,''    ,''   ,'deg'   ,'m/s'  ,'-'   ,
	'W/m2'  ,'W/m2','W/m2'  ,'W/m2','m'     ,'hPa']

Header_met_all = ['date','U1','WD1','T1','RH1','SWd','SWu','LWd','LWu','sh','Pa','solz','sola','SWd_slope','albedo','daily_integrated_albedo','sensor_height1','Ts','solz_slope']

Header_mod = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse']

Unit_mod =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1']


#======================================
#========== main
def main():
	#===== set path
	indir_met = '../data/met_lv3'
	indir_mod = '../%s/out_else'%model
	outdir = '../fig/%s'%model

	#=== set plot param
	plt.rcParams['font.size'] = 15
	fig, axes = plt.subplots(4, 3, 
			gridspec_kw=dict(width_ratios=[1,1,1],height_ratios=[1,2,1,2],wspace=0.1,hspace=0.2),
			figsize=(22,20))

	y_list = [2012,2014,2017]
	fig_list = ['fig5a','fig5b']

	for i in range(6):
		if i < 3:
			figNm = fig_list[0]
		else:
			figNm = fig_list[1]
			i = i -3
		year = y_list[i]	
		print figNm,year

		#===== read csv data
		if year == 2012:
			fileN_met = 'SIGMA-B_LT20120720-_Lv1_3'
			fileN_met_all = 'SIGMA_AWS_SiteB_UTC20120720-20121231_Lv1_3'
			if figNm == 'fig5a':
				fileN_mod = 'ch_hourly_%d_v3_at0_di50_id100_ai70_fig5'%year
			elif figNm == 'fig5b':
				fileN_mod = 'ch_hourly_%d_v3_at39_di30_id30_ai57_fig5'%year
		elif year == 2014:
			fileN_met = 'SIGMA-B_LT20140701-_Lv1_3'
			fileN_met_all = 'SIGMA_AWS_SiteB_UTC20140101-20141231_Lv1_3'
			if figNm == 'fig5a':
				fileN_mod = 'ch_hourly_%d_v3_at0_di50_id100_ai70_fig5'%year
			elif figNm == 'fig5b':
				fileN_mod = 'ch_hourly_%d_v3_at39_di51_id186_ai57_fig5'%year
		elif year == 2017:
			fileN_met = 'SIGMA-B_LT20170701-_Lv1_3'
			fileN_met_all = 'SIGMA_AWS_SiteB_UTC20170101-20171231_Lv1_3'
			if figNm == 'fig5a':
				fileN_mod = 'ch_hourly_%d_v3_at0_di50_id100_ai70_fig5'%year
			elif figNm == 'fig5b':
				fileN_mod = 'ch_hourly_%d_v3_at39_di41_id149_ai42_fig5'%year

		data_met = rd_csv(indir_met,fileN_met,Header_met,'met')
		data_met[np.where(data_met<-99) ] = np.nan

		data_met_all = rd_csv(indir_met,fileN_met_all,Header_met_all,'met_all')
		data_met_all[np.where(data_met_all<-99) ] = np.nan
		if year in [2012]:
			ts = data_met_all[-2,:]
		else:
			ts = data_met_all[-2,4343:-1]	# Ts 0701~1231

		data_mod = rd_csv(indir_mod,fileN_mod,Header_mod,'mod')

		#=== plot figure
		if figNm == 'fig5a':
			make_subplt_met(axes[0][i],year,data_met,Header_met,ts,i,'fig5a')
			make_subplt_heat(axes[1][i],year,data_mod,Header_mod,i,'fig5a')
		elif figNm == 'fig5b':
			make_subplt_met(axes[2][i],year,data_met,Header_met,ts,i,'fig5b')
			make_subplt_heat(axes[3][i],year,data_mod,Header_mod,i,'fig5b')

	#===== save or show figure
	figN = 'fig5'

	if figSave=='on':
		print'### save figure ###'
		if os.path.isdir('%s'%outdir) == False: os.makedirs('%s'%outdir)
		plt.savefig('%s/%s.png'%(outdir,figN),format='png',bbox_inches="tight")
	else:
		plt.show()
	print 'Out dir = %s'%outdir
	print 'Figure name = %s.png'%(figN)

#======================================
#===== set path & read model data
def rd_csv(indir,fileN,Header,opt):

	infile = '%s/%s.csv'%(indir,fileN)
	print 'infile',infile

	if os.path.exists(infile) and opt == 'met':
		df = pd.read_csv(infile,names=Header,encoding='UTF-8')
	elif os.path.exists(infile) and opt in ['met_all','mod']:
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		exit()

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print data.shape

	if opt == 'met_all':
		n = 1
	else:
		n = 0

	for i in range(n,len(Header)):
		data[i,:] = np.array(df[Header[i]].values.tolist())
		#print "%s mean, min, max = %.02f, %.02f, %.02f"%(Header[i],np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))
	
	return data

#======================================
#===== plot figure
def make_subplt_met(axes,year,d,Header,ts,i,figNm):

	#print d.shape
	Header_idx = ['atemp','swdown','ws','rh']
	if figNm == 'fig5a':
		varList =  ['Temperatures','Solar radiation','Wind speed','Relative humidity']
	elif figNm == 'fig5b':
		varList =  ['Air temperature','Solar radiation','Wind speed','Relative humidity']
	unitList = ['K','W $\mathregular{m^{-2}}$','m $\mathregular{s^{-1}}$','%']
	yminList = [-5+273.15,0,0,40]
	ymaxList = [10+273.15,700,15,100]

	#=== plot
	hour_idx = 3
	doy_idx = 4

	print varList[0]
	var_idx = Header.index(Header_idx[0])

	#= plt var
	if figNm == 'fig5a':
		axes.plot((d[doy_idx,:]+d[hour_idx,:]/24),d[var_idx,:]+273.15,color='red',linestyle='-',ls=2,label='$T_{a}$')
		axes.plot((d[doy_idx,:]+d[hour_idx,:]/24),ts+273.15,color='darkorange',linestyle='--',ls=2,label='$T_{s}$')
	elif figNm == 'fig5b':
		axes.plot((d[doy_idx,:]+d[hour_idx,:]/24),d[var_idx,:]+3.92+273.15,color='red',linestyle='-',ls=2)

	#= set label
	axes.set_xlim(xmin=185)
	axes.set_xlim(xmax=216)
	axes.set_ylim(ymin=yminList[0])
	axes.set_ylim(ymax=ymaxList[0])
	axes.xaxis.set_ticklabels([])
	axes.axhline(y=273.15,color='black',ls=':',linewidth=1.5)

	if i == 0:
		if figNm == 'fig5a':
			axes.legend(loc='upper left',fontsize=22)
			#axes.text(0.15,0.30,'No data',transform=axes.transAxes,size=24)
		#elif figNm == 'fig5b':
			#axes.text(0.15,0.50,'No data',transform=axes.transAxes,size=24)
		axes.axvline(x=201,ymin=0,ymax=1,color='black',ls='--')
		axes.set_ylabel('%s\n(%s)'%(varList[0],unitList[0]),fontsize=24)
		axes.tick_params(axis='y',labelsize=20)
	elif i > 0:
		axes.yaxis.set_ticklabels([])

#======================================
#===== plot figure
def make_subplt_heat(axes,year,d,Header,i,figNm):

	Header_idx = ['nsr_ice','nlr_ice','sh_ice','lh_ice','sr']
	varList =  ['Air temperature','Solar radiation','Wind speed','Relative humidity']
	unitList = ['${}^\circ\mathrm{C}$','W $\mathregular{m^{-2}}$','m $\mathregular{s^{-1}}$','%']

	#=== cal daily mean
	d_daily = np.zeros((d.shape[0],d.shape[1]/24))
	print d_daily.shape

	for v in range(d_daily.shape[0]):
		for t in range(d_daily.shape[1]):
			if v == 0:
				d_daily[v,t] = int(np.mean(d[v,t*24:(t+1)*24]))
			else:
				d_daily[v,t] = np.mean(d[v,t*24:(t+1)*24]) 

	if year == 2012:
		net_sw = d_daily[Header.index(Header_idx[0]),:]
		net_lw = d_daily[Header.index(Header_idx[1]),:]
		sh = d_daily[Header.index(Header_idx[2]),:]
		lh = d_daily[Header.index(Header_idx[3]),:]
		swdown = d_daily[Header.index(Header_idx[4]),:]
		doy = d_daily[0,:]
		d_bar = np.zeros((3,d_daily.shape[1]))
	else:
		net_sw = d_daily[Header.index(Header_idx[0]),4:]
		net_lw = d_daily[Header.index(Header_idx[1]),4:]
		sh = d_daily[Header.index(Header_idx[2]),4:]
		lh = d_daily[Header.index(Header_idx[3]),4:]
		swdown = d_daily[Header.index(Header_idx[4]),4:]
		doy = d_daily[0,4:]
		d_bar = np.zeros((3,d_daily.shape[1]-4))

	#=== + - bar
	d_bar[0] = net_sw + net_lw
	d_bar[1] = sh
	d_bar[2] = lh

	posi = d_bar.clip(min=0)
	cum_posi = np.cumsum(posi,axis=0)
	cum_d1 = np.zeros(d_bar.shape)
	cum_d1[1:] = cum_posi[:-1]

	nega = d_bar.clip(max=0)
	cum_nega = np.cumsum(nega,axis=0)
	cum_d2 = np.zeros(d_bar.shape)
	cum_d2[1:] = cum_nega[:-1]

	row_mask = (d_bar<0)
	cum_d1[row_mask] = cum_d2[row_mask]
	cum_d = cum_d1

	axes.bar(doy,d_bar[0,:],bottom=cum_d[0,:],color='darkgoldenrod',edgecolor='black',label='Net radiation')
	axes.bar(doy,d_bar[1,:],bottom=cum_d[1,:],color='olivedrab',edgecolor='black',label='Sensible heat')
	axes.bar(doy,d_bar[2,:],bottom=cum_d[2,:],color='lightslategrey',edgecolor='black',label='Latent heat')
	axes.axhline(y=0,color='black',ls='-',lw=0.5)

	axes.set_xlim(xmin=185)
	axes.set_xlim(xmax=216)
	axes.tick_params(axis='x',labelsize=20)

	if figNm == 'fig5a':
		axes.set_ylim(ymax=150)
		axes.set_ylim(ymin=-75)
	elif figNm == 'fig5b':
		axes.set_ylim(ymax=350)
		axes.set_ylim(ymin=-50)
		axes.set_xlabel("Day of year %d"%year,fontsize=24)

	#axes.text(0.02,0.9,'(a)',transform=axes.transAxes,size=20)

	if i == 0:
		axes.set_ylabel('Energy balance\n(W $\mathregular{m^{-2}}$)',fontsize=24)
		axes.legend(loc='upper left',fontsize=22)
		axes.tick_params(axis='y',labelsize=20)
		axes.axvline(x=201,ymin=0,ymax=0.65,color='black',ls='--')
		#axes.text(0.15,0.45,'No data',transform=axes.transAxes,size=24)
	elif i > 0:
		axes.yaxis.set_ticklabels([])

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'May 2023'

