#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys
import scipy.stats
from sklearn.metrics import mean_squared_error

#======================================
#========== set param
figSave = 'off'
model = 'cryho_disclose_v1'

Year_list = [2014,2014,2014,2014,2012,2017,2017,2017]
Site_list = ['QA-S1','QA-S2','QA-S3','QA-S4','QA-S3','QA-S1','QA-S2','QA-S3']

fileN_list = ['2014_v3_at54_di39_id141_ai68','2014_v3_at39_di51_id186_ai57',
'2014_v3_at21_di33_id86_ai40','2014_v3_at13_di43_id110_ai41','2012_v3_at21_di33_id38_ai40',
'2017_v3_at54_di48_id168_ai56','2017_v3_at39_di41_id149_ai42','2017_v3_at21_di38_id111_ai24']

fileN_list_min = ['2014_v3_at54_di39_id141_ai65','2014_v3_at39_di51_id186_ai54',
'2014_v3_at21_di33_id86_ai33','2014_v3_at13_di43_id110_ai33','2012_v3_at21_di33_id38_ai33',
'2017_v3_at54_di48_id168_ai45','2017_v3_at39_di41_id149_ai38','2017_v3_at21_di38_id111_ai18']

fileN_list_max = ['2014_v3_at54_di39_id141_ai71','2014_v3_at39_di51_id186_ai60',
'2014_v3_at21_di33_id86_ai47','2014_v3_at13_di43_id110_ai49','2012_v3_at21_di33_id38_ai47',
'2017_v3_at54_di48_id168_ai67','2017_v3_at39_di41_id149_ai46','2017_v3_at21_di38_id111_ai30']

#======================================
#========== main
def main():
	#===== set path
	indir_mod = '../%s/out_cal_obs'%(model)
	outdir = '../fig/%s'%(model)

	doy_r,dp_r_ave,dp_r_sd = [0]*8,[0]*8,[0]*8
	doy_c,dp_c_ave,dp_c_sd = [0]*8,[0]*8,[0]*8
	doy_m,dp_m = [0]*8,[0]*8
	doy_m_min,dp_m_min = [0]*8,[0]*8
	doy_m_max,dp_m_max = [0]*8,[0]*8

	#===== read csv data
	print'### read obs data (csv) ###'
	for n in range(len(Year_list)):
		year = Year_list[n]
		site = Site_list[n]
		print year,site
		indir_obs = '../data/CHdepth'
		fileN_m = fileN_list[n]
		fileN_m_min = fileN_list_min[n]
		fileN_m_max = fileN_list_max[n]
		doy_r[n],dp_r_ave[n],dp_r_sd[n] = rd_csv_ruler(indir_obs,site,year)
		doy_c[n],dp_c_ave[n],dp_c_sd[n] = rd_csv_camera(indir_obs,site,year)
		doy_m[n],dp_m[n]                = rd_csv_mod(indir_mod,fileN_m)
		doy_m_min[n],dp_m_min[n]        = rd_csv_mod(indir_mod,fileN_m_min)
		doy_m_max[n],dp_m_max[n]        = rd_csv_mod(indir_mod,fileN_m_max)

	print'### plot ###'
	#=== figure
	sbplot(outdir,doy_r,dp_r_ave,dp_r_sd,doy_c,dp_c_ave,dp_c_sd,doy_m,dp_m,dp_m_min,dp_m_max,'fig7')
	sbplot(outdir,doy_r,dp_r_ave,dp_r_sd,doy_c,dp_c_ave,dp_c_sd,doy_m,dp_m,dp_m_min,dp_m_max,'fig8')

#======================================
#===== read observational data using ruler (CSV)
def rd_csv_ruler(indir,site_obs,year_obs):

	Tab_list = ['date','doy','diamL_ave','diamL_sd','diamS_ave','diamS_sd','depthA_ave','depthA_sd','depthW_ave','depthW_sd']

	if year_obs == 2012:
		print "no ovs data skip"
		doy,dp_ave,dp_sd = -999,-999,-999
		return doy,dp_ave,dp_sd
	else:
		obs_file = '%s/%d%s_CH_season.csv'%(indir,year_obs,site_obs)

	df = pd.read_csv(obs_file,encoding='UTF-8')

	#=== read csv data
	doy = df[Tab_list[1]].values.tolist()
	dp_ave = df[Tab_list[6]].values.tolist()
	dp_sd = df[Tab_list[7]].values.tolist()
	
	print "### %s (%d) ###"%(site_obs,year_obs)
	print doy
	print dp_ave
	print dp_sd

	return doy,dp_ave,dp_sd

#======================================
#===== read observational data using interval camera (CSV)
def rd_csv_camera(indir,site_obs,year_obs):

	if year_obs == 2014 and site_obs == 'QA-S2':
		Tab_list = ['PhotoNo','date','doy','depthA']
		obs_file = '%s/%d%s_CH_monitor_season.csv'%(indir,year_obs,site_obs)
		df = pd.read_csv(obs_file,encoding='UTF-8')
		doy = df[Tab_list[2]].values.tolist()
		dp_ave = df[Tab_list[3]].values.tolist()
		dp_sd = -999
	else:
		print "no ovs data skip"
		doy,dp_ave,dp_sd = -999,-999,-999
		return doy,dp_ave,dp_sd
	
	return doy,dp_ave,dp_sd

#======================================
#===== set path & read model data
def rd_csv_mod(indir,fileN):

	Header = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse']

	Unit =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1']

	if len(fileN) == 0:
		infile = '%s/ch_hourly.csv'%(indir)
	else:
		infile = '%s/ch_hourly_%s.csv'%(indir,fileN)
	print 'infile',infile

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		return data

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print data.shape

	for i in range(len(Header)):
		data[i,:] = np.array(df[Header[i]].values.tolist())
		print Header[i],"mean, min, max = %.02f, %.02f, %.02f"%(np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))

	doy = data[0,:]
	idx = Header.index('cryoconite_depth')
	ch_depth = data[idx,:]

	return doy,ch_depth

#======================================
#===== cal correlation
def cal_corr(doy_c,dpA_c,doy_m,dpA_m):
	data_corr = []

	for n in range(len(doy_c)):	
		#print doy_c[n]
		idx = np.abs(np.asarray(doy_m) - doy_c[n]).argmin()
		#print doy_m[idx]

		data_corr = np.append(data_corr,dpA_m[idx])

	bad = ~np.logical_or(np.isnan(data_corr),np.isnan(dpA_c))	# remove nan in both data
	d1 = np.compress(bad,data_corr)
	d_ref = np.compress(bad,dpA_c)

	R,P = scipy.stats.pearsonr(d1,d_ref)
	R2 = R**2
	print 'R, P, R^2 (vs. obs)= ',R,P,R2

	return R,P,R2

#======================================
#===== cal root mean square
def cal_rmse(doy_c,dpA_c,doy_m,dpA_m):
	data_rmse = []

	for n in range(len(doy_c)):	
		#print doy_c[n]
		idx = np.abs(np.asarray(doy_m) - doy_c[n]).argmin()
		#print doy_m[idx]

		data_rmse = np.append(data_rmse,dpA_m[idx])

	bad = ~np.logical_or(np.isnan(data_rmse),np.isnan(dpA_c))	# remove nan in both data
	d1 = np.compress(bad,data_rmse)
	d_ref = np.compress(bad,dpA_c)

	RMSE = np.sqrt(mean_squared_error(d1,d_ref))
	print 'RMSE (vs. obs)=',RMSE
	
	return RMSE

#======================================
#===== plot figure
def sbplot(outdir,doy_r_l,dp_r_ave_l,dp_r_sd_l,doy_c_l,dp_c_ave_l,dp_c_sd_l,doy_m_l,dp_m_l,dp_m_min_l,dp_m_max_l,opt):

	label1 = 'Obs (ruler)'
	label2 = 'Obs (camera)'
	label3 = 'Model'

	#=== set plot param
	fig, axes = plt.subplots(2, 2, figsize=(15,13))
	plt.subplots_adjust(wspace=0.10,hspace=0.10) # adjust plot space
	plt.tick_params(labelsize=18)

	#=== plot model result
	y_min, y_max = 0, 0.25
	doy_min, doy_max = 180,220

	i,j = 0,0

	for n in range(4):
		print n,i,j

		if opt == 'fig7':
			s = n
		elif opt == 'fig8':
			s = n+4

		#== plot result (model) mm -> m
		dp_m_l[s] = dp_m_l[s] * 0.001
		dp_m_min_l[s] = dp_m_min_l[s] * 0.001
		dp_m_max_l[s] = dp_m_max_l[s] * 0.001

		axes[i][j].plot(doy_m_l[s],dp_m_l[s],color='black',linestyle='-',lw=2,label=label3)
		axes[i][j].fill_between(doy_m_l[s],dp_m_min_l[s],dp_m_max_l[s],facecolor='black',alpha=0.15)


		#== set axis
		axes[i][j].set_xlim(xmin=doy_min)
		axes[i][j].set_xlim(xmax=doy_max)
		xlabels = np.arange(doy_min,doy_max+1,10)
		axes[i][j].set_xticks(xlabels)

		if j == 0:
			axes[i][j].set_ylabel('Depth of cryoconite hole (m)',fontsize=23)
			axes[i][j].tick_params(axis='y', which='major',labelsize=16)
		else:
			axes[i][j].axes.yaxis.set_ticklabels([])
		if i == 1:
			axes[i][j].set_xlabel('Day of year',fontsize=23)
			axes[i][j].tick_params(axis='x', which='major',labelsize=16)
		else:
			axes[i][j].axes.xaxis.set_ticklabels([])

		axes[i][j].set_ylim(ymin=y_min)
		axes[i][j].set_ylim(ymax=y_max)
		axes[i][j].invert_yaxis()

		#== plot result (ruler)  cm -> m
		if Year_list[s] in [2014,2017]:
			dp_r_ave = np.array(dp_r_ave_l[s]) * 0.01
			dp_r_sd = np.array(dp_r_sd_l[s]) * 0.01
			axes[i][j].errorbar(doy_r_l[s],dp_r_ave,yerr=[dp_r_sd,dp_r_sd],color="grey",
			ls='',lw=1.5,markersize=10,marker='o',ecolor='grey',capsize=2,label=label1)

		#== plot result (camera) cm -> m
		if Year_list[s] in [2014]:
			dp_c_ave = np.array(dp_c_ave_l[s]) * 0.01

		if Year_list[s] == 2014 and Site_list[s] == 'QA-S2':
			axes[i][j].plot(doy_c_l[s],dp_c_ave,color='grey',linestyle='-',lw=2,label=label2)
			R,P,R2 = cal_corr(doy_c_l[s],dp_c_ave,doy_m_l[s],dp_m_l[s])
			RMSE = cal_rmse(doy_c_l[s],dp_c_ave,doy_m_l[s],dp_m_l[s])

		if s == 0:
			axes[i][j].text(182,0.025,'a) Site 1',color='black',size=24)
			axes[i][j].text(182,0.050,'$\Phi$ = 0.04 m',size=20)
			axes[i][j].text(182,0.068,'$D_{0}$ = 0.14 m',size=20)
			axes[i][j].text(182,0.086,r'$\alpha_{i}$ = 0.68$\pm$0.03',size=20)
			#axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +5.44${}^\circ\mathrm{C}$',size=20)
			axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +5.44 K',size=20)
		elif s == 1:
			axes[i][j].text(182,0.025,'b) Site 2',color='black',size=24)
			axes[i][j].text(182,0.050,'$\Phi$ = 0.05 m',size=20)
			axes[i][j].text(182,0.068,'$D_{0}$ = 0.19 m',size=20)
			axes[i][j].text(182,0.086,r'$\alpha_{i}$ = 0.57$\pm$0.03',size=20)
			axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +3.92 K',size=20)
		elif s == 2:
			axes[i][j].text(182,0.025,'c) Site 3',color='black',size=24)
			axes[i][j].text(182,0.050,'$\Phi$ = 0.03 m',size=20)
			axes[i][j].text(182,0.068,'$D_{0}$ = 0.09 m',size=20)
			axes[i][j].text(182,0.086,r'$\alpha_{i}$ = 0.40$\pm$0.07',size=20)
			axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +2.12 K',size=20)
		elif s == 3:
			axes[i][j].text(182,0.025,'d) Site 4',color='black',size=24)
			axes[i][j].text(182,0.050,'$\Phi$ = 0.04 m',size=20)
			axes[i][j].text(182,0.068,'$D_{0}$ = 0.11 m',size=20)
			axes[i][j].text(182,0.086,r'$\alpha_{i}$ = 0.41$\pm$0.08',size=20)
			axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +1.34 K',size=20)
		elif s == 4:
			axes[i][j].text(182,0.025,'a) Site 3 (2012)',color='black',size=24)
			axes[i][j].text(182,0.050,'$\Phi$ = 0.03 m',size=20)
			axes[i][j].text(182,0.068,'$D_{0}$ = 0.04 m',size=20)
			axes[i][j].text(182,0.086,r'$\alpha_{i}$ = 0.40$\pm$0.07',size=20)
			axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +2.12 K',size=20)
			axes[i][j].axvline(x=206.5,color='grey',ls='--')
			axes[i][j].axvline(x=208.5,color='grey',ls='--')
		elif s == 5:
			axes[i][j].text(182,0.025,'b) Site 1 (2017)',color='black',size=24)
			axes[i][j].text(183,0.050,'$\Phi$ = 0.05 m',size=20)
			axes[i][j].text(183,0.068,'$D_{0}$ = 0.17 m',size=20)
			axes[i][j].text(183,0.086,r'$\alpha_{i}$ = 0.56$\pm$0.11',size=20)
			axes[i][j].text(183,0.104,'$\Delta$$T_{a}$ = +5.44 K',size=20)
		elif s == 6:
			axes[i][j].text(182,0.025,'c) Site 2 (2017)',color='black',size=24)
			axes[i][j].text(182,0.050,'$\Phi$ = 0.04 m',size=20)
			axes[i][j].text(182,0.068,'$D_{0}$ = 0.15 m',size=20)
			axes[i][j].text(182,0.086,r'$\alpha_{i}$ = 0.42$\pm$0.04',size=20)
			axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +3.92 K',size=20)
		elif s == 7:
			axes[i][j].text(182,0.025,'d) Site 3 (2017)',color='black',size=24)
			#axes.text(182,25,'$\mathregular{R^{2}}$ = %0.2f'%R2,color='black',size=18)
			#axes.text(182,40,'RMSE = %1.2f mm'%RMSE,color='black',size=18)
			axes[i][j].text(182,0.050,'$\Phi$ = 0.04 m',size=20)
			axes[i][j].text(182,0.068,'$D_{0}$ = 0.11 m',size=20)
			axes[i][j].text(182,0.086,r'$\alpha_{i}$ = 0.24$\pm$0.06',size=20)
			axes[i][j].text(182,0.104,'$\Delta$$T_{a}$ = +2.12 K',size=20)

		handles, labels = axes[i][j].get_legend_handles_labels()
		axes[i][j].legend(handles[::-1],labels[::-1],loc='lower right',fontsize=20)	# invert legend order
		j = j + 1
		if j == 2:
			i = i + 1
			j = 0

	if opt == 'fig7': 
		figName = 'fig7'
	elif opt == 'fig8': 
		figName = 'fig8'

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s.png'%(outdir,figName),bbox_inches="tight")
		print 'save %s/%s.png'%(outdir,figName)
	else:
		print '%s/%s.png'%(outdir,figName)
		plt.show()
	plt.close()
#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'May 2023'
