#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import sys
import os

figSave = 'off'
model = 'cryho_disclose_v1'

fileN_ctl = 'ch_hourly_2014_st_sr1'
fileN_st = 'ch_hourly_2014_st'

stList = ['at','sr','d0','phi','ai','ac','kisf','kidf','zan','can']

Header = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse','src_diffuse_t']

Unit =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1','W/m2']

#atList = ['at0.92','at1.92','at2.92','at4.92','at5.92','at6.92']
atList = ['at6.92','at5.92','at4.92','at2.92','at1.92','at0.92']
srList = ['sr2','sr3']
d0List = ['d00','d050','d0100','d0150','d0200']
phiList = ['phi10','phi30','phi70','phi90','phi110']
aiList = ['ai0.3','ai0.4','ai0.5','ai0.6','ai0.7']
#acList = ['ac0.05','ac0.15','ac0.2','ac0.25','ac0.3']
acList = ['ac0.3','ac0.25','ac0.2','ac0.15','ac0.05']
#kisfList = ['kisf0.25','kisf0.5','kisf2','kisf4']
kisfList = ['kisf4','kisf2','kisf0.5','kisf0.25']
kidfList = ['kidf4','kidf2','kidf0.5','kidf0.25']
zanList = ['zan75','zan60','zan45','zan30','zan15','zan0']
canList = ['can15','can30','can45','can60','can75','can90']

#======================================
#========== main
def main():
	#===== set path
	indir_ctl = '../%s/out_stest'%model
	indir_st = '../%s/out_stest'%model
	outdir = '../fig/%s'%model

	#===== read csv data (control)
	d_ctl = rd_csv(indir_ctl,fileN_ctl,Header,'','ctl')

	at_all = np.zeros((len(atList),d_ctl.shape[0],d_ctl.shape[1]))
	sr_all = np.zeros((len(srList),d_ctl.shape[0],d_ctl.shape[1]))
	d0_all = np.zeros((len(d0List),d_ctl.shape[0],d_ctl.shape[1]))
	phi_all = np.zeros((len(phiList),d_ctl.shape[0],d_ctl.shape[1]))
	ai_all = np.zeros((len(aiList),d_ctl.shape[0],d_ctl.shape[1]))
	ac_all = np.zeros((len(acList),d_ctl.shape[0],d_ctl.shape[1]))
	kidf_all = np.zeros((len(kidfList),d_ctl.shape[0],d_ctl.shape[1]))
	kisf_all = np.zeros((len(kisfList),d_ctl.shape[0],d_ctl.shape[1]))
	zan_all = np.zeros((len(zanList),d_ctl.shape[0],d_ctl.shape[1]))
	can_all = np.zeros((len(zanList),d_ctl.shape[0],d_ctl.shape[1]))

	#===== read csv data (sensitivity test)
	for i in range(len(atList)):
		at_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,atList[i],'at')
	for i in range(len(srList)):
		sr_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,srList[i],'sr')
	for i in range(len(d0List)):
		d0_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,d0List[i],'d0')
	for i in range(len(phiList)):
		phi_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,phiList[i],'phi')
	for i in range(len(aiList)):
		ai_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,aiList[i],'ai')
	for i in range(len(acList)):
		ac_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,acList[i],'ac')
	for i in range(len(kidfList)):
		kidf_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,kidfList[i],'kidf')
	for i in range(len(kisfList)):
		kisf_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,kisfList[i],'kisf')
	for i in range(len(zanList)):
		zan_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,zanList[i],'zan')
	for i in range(len(canList)):
		can_all[i,:,:] = rd_csv(indir_st,fileN_st,Header,canList[i],'can')

	#=== plot figure
	make_subplt_st(outdir,d_ctl,at_all,sr_all,d0_all,phi_all,ai_all,ac_all,kidf_all,kisf_all,zan_all,can_all)

#======================================
#===== set path & read model data
def rd_csv(indir,fileN,Header,st,opt):

	if opt == 'ctl':
		infile = '%s/%s.csv'%(indir,fileN)
	else:
		print st
		infile = '%s/%s_%s.csv'%(indir,fileN,st)

	print 'infile',infile

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		exit()

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print opt,data.shape

	for i in range(len(Header)):
		data[i,:] = np.array(df[Header[i]].values.tolist())
		print "%s mean, min, max = %.02f, %.02f, %.02f"%(Header[i],np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))
		
	return data

#======================================
#===== plot figure
def make_subplt_st(outdir,d_ctl,d_at,d_sr,d_d0,d_phi,d_ai,d_ac,d_kisf,d_kidf,d_zan,d_can):

	#=== set plot param
	fig, axes = plt.subplots(5, 2, figsize=(18,22))
	plt.subplots_adjust(wspace=0.05,hspace=0.15) # adjust plot space

	doy = d_ctl[0,:]

	d_list = [d_at,d_sr,d_d0,d_phi,d_ai,d_ac,d_kisf,d_kidf,d_zan,d_can]

	#=== plot
	i,j = 0,0
	for n in range(len(stList)):
		print stList[n]
		st = stList[n]
		plt_st(doy,d_ctl,d_list[n],st,axes[i][j])

		i = i + 1
		if i == 5:
			j = j + 1
			i = 0

	#=== set label axis
	#fig_list = ['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)']
	fig_list2 = ['a) $\mathregular{T_{a}}$-exp','b) $\mathregular{R_{S}}$-exp','c) $\mathregular{D_{0}}$-exp','d) $\mathregular{\Phi}$-exp',r'e) $\alpha_{i}$-exp',r'f) $\alpha_{c}$-exp','g) $\mathregular{\kappa_{d}}$-exp','h) $\mathregular{\kappa_{f}}$-exp',r'i) $\theta_{z}$-exp',r'j) $\theta_{c}$-exp']

	i,j = 0,0
	for n in range(10):
		axes[i][j].set_xlim(xmin=182)
		axes[i][j].set_xlim(xmax=235)

		axes[i][j].set_ylim(ymin=0)
		axes[i][j].set_ylim(ymax=0.35)
		axes[i][j].invert_yaxis()

		axes[i][j].tick_params(axis='y',labelsize=20)
		axes[i][j].tick_params(axis='x',labelsize=20)
		if i < 4:
			axes[i][j].xaxis.set_ticklabels([])
		elif i == 4:
			axes[i][j].set_xlabel("Day of year",fontsize=24)
	
		if j > 0:
			axes[i][j].yaxis.set_ticklabels([])
		else:
			axes[i][j].set_ylabel("Cryoconite hole\ndepth (m)",fontsize=24)

		axes[i][j].text(0.030,0.10,fig_list2[n],transform=axes[i][j].transAxes,size=24)

		axes[i][j].legend(loc='lower right',fontsize=19)

		i = i + 1
		if i == 5:
			j = j + 1
			i = 0

	#===== save or show figure
	figN = 'fig9'

	if figSave=='on':
		print'### save figure ###'
		if os.path.isdir('%s'%outdir) == False: os.makedirs('%s'%outDir)
		plt.savefig('%s/%s.png'%(outdir,figN),format='png',bbox_inches="tight")
	else:
		plt.show()
	print 'Out dir = %s'%outdir
	print 'Figure name = %s.png'%(figN)

#======================================
#===== plot figure
def plt_st(doy,ctl,d,st,axes):

	#colorList = ['blue','aqua','lime','orange','red']
	colorList = ['red','orange','lime','aqua','blue']
	#colorList_at = ['purple','blue','aqua','lime','orange','red']
	colorList_at = ['red','orange','lime','aqua','blue','purple']

	if st == "at":
		Label = ['+3','+2','+1','-1','-2','-3']
		#unit = '${}^\circ\mathrm{C}$'
		unit = 'K'
	elif st == "d0":
		Label = ['0.00','0.05','0.10','0.15','0.20']
		unit = 'm'
		ctl_v = '0.19'
	elif st == "phi":
		Label = ['0.01','0.03','0.07','0.09','0.11']
		unit = 'm'
		ctl_v = '0.05'
	elif st == "ai":
		Label = ['0.30','0.40','0.50','0.60','0.70']
		unit = ''
		ctl_v = '0.57'
	elif st == "ac":
		Label = ['0.30','0.25','0.20','0.15','0.05']
		unit = ''
		ctl_v = '0.10'
	elif st == "kisf":
		Label = ['x4.00','x2.00','x0.50','x0.25']
		unit = ''
		ctl_v = 'x1.00'
	elif st == "kidf":
		Label = ['x4.00','x2.00','x0.50','x0.25']
		unit = ''
		ctl_v = 'x1.00'
	elif st == "sr":
		Label = ['Direct','Diffuse']
		unit = ''
		ctl_v = 'Direct and Diffuse'
	elif st == "zan":
		Label = [u'75\N{DEGREE SIGN}',u'60\N{DEGREE SIGN}',u'45\N{DEGREE SIGN}',u'30\N{DEGREE SIGN}',u'15\N{DEGREE SIGN}',u'0\N{DEGREE SIGN}']
		unit = ''
		ctl_v = 'Control'
	elif st == "can":
		Label = [u'15\N{DEGREE SIGN}',u'30\N{DEGREE SIGN}',u'45\N{DEGREE SIGN}',u'60\N{DEGREE SIGN}',u'75\N{DEGREE SIGN}',u'90\N{DEGREE SIGN}']
		unit = ''
		ctl_v = 'Control'

	print 'exp %s'%st
	for i in range(len(Label)):
		#ch-depth_idx = 23
		# mm -> m by using 0.001

		if i == 3 and st in ["at"]:
			axes.plot(doy,ctl[23,:]*0.001,color='black',linestyle='-',ls=1.5,
					label='+0 %s'%(unit))
		elif i == 3 and st in ["ai"]:
			axes.plot(doy,ctl[23,:]*0.001,color='black',linestyle='-',ls=1.5,
					label='%s %s'%(ctl_v,unit))
		elif i == 4 and st in ["ac","d0"]:
			axes.plot(doy,ctl[23,:]*0.001,color='black',linestyle='-',ls=1.5,
					label='%s %s'%(ctl_v,unit))
		elif i == 2 and st in ["phi","kisf","kidf"]:
			axes.plot(doy,ctl[23,:]*0.001,color='black',linestyle='-',ls=1.5,
					label='%s %s'%(ctl_v,unit))
		elif st in ['zan'] and i == 1:
			axes.plot(doy,ctl[23,:]*0.001,color='black',linestyle='-',ls=1.5,
					label='Control')
		elif st in ['can'] and i == 3:
			axes.plot(doy,ctl[23,:]*0.001,color='black',linestyle='-',ls=1.5,
					label='Control')
		elif st == 'sr' and i == 1:
			axes.plot(doy,ctl[23,:]*0.001,color='black',linestyle='-',ls=1.5,
					label='Control')
			axes.plot(doy,d[i,23,:]*0.001,color=colorList[4],linestyle='-',ls=1.5,
						label='%s %s'%(Label[i],unit))
			continue

		#= plt var
		if st in ["at","zan","can"]:
			axes.plot(doy,d[i,23,:]*0.001,color=colorList_at[i],linestyle='-',ls=1.5,
						label='%s %s'%(Label[i],unit))
		else:
			axes.plot(doy,d[i,23,:]*0.001,color=colorList[i],linestyle='-',ls=1.5,
						label='%s %s'%(Label[i],unit))

#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'May. 2023'

