#!/usr/bin/env python2.7
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys
import scipy.stats
from sklearn.metrics import mean_squared_error
from sklearn.metrics import mean_absolute_error

#======================================
#========== set param
figSave = 'on'
Site = 'QA-S2'	# output site (Qaanaaq Site 2)
Year = 2014	# output year
model = 'cryho_disclose_v1'

fileN_m = '%d_v3_monitor_at39_di51_id117_ai62'%Year  # 2014 Site2 monitoring site

doy_min, doy_max = 180,220

#======================================
#========== main
def main():
	#===== set path
	indir_obs = '../data/CHdepth'
	indir_mod = '../%s/out_else'%(model)
	outdir = '../fig/%s'%(model)

	#===== read csv data
	print'### read obs data (csv) ###'
	doy_r,dp_r_ave,dp_r_sd = rd_csv_ruler(indir_obs,Site,Year)
	doy_c,dp_c_ave,dp_c_sd = rd_csv_camera(indir_obs,Site,Year)
	doy_m,dp_m             = rd_csv_mod(indir_mod,fileN_m)

	print'### plot %s ###'%Site
	#=== figure
	plot(outdir,doy_r,dp_r_ave,dp_r_sd,doy_c,dp_c_ave,dp_c_sd,doy_m,dp_m)

#======================================
#===== read observational data using ruler (CSV)
def rd_csv_ruler(indir,site_obs,year_obs):

	Tab_list = ['date','doy','diamL_ave','diamL_sd','diamS_ave','diamS_sd','depthA_ave','depthA_sd','depthW_ave','depthW_sd']

	if year_obs == 2012:
		print "no ovs data skip"
		doy,dp_ave,dp_sd = -999,-999,-999
		return doy,dp_ave,dp_sd
	elif year_obs == 2017 and Site == 'QA-S3':
		obs_file = '%s/%d%s_CH_season_v2.csv'%(indir,year_obs,site_obs)
	else:
		obs_file = '%s/%d%s_CH_season.csv'%(indir,year_obs,site_obs)

	df = pd.read_csv(obs_file,encoding='UTF-8')

	#=== read csv data
	doy = df[Tab_list[1]].values.tolist()
	dp_ave = df[Tab_list[6]].values.tolist()
	dp_sd = df[Tab_list[7]].values.tolist()
	
	print "### %s (%d) ###"%(site_obs,year_obs)
	print doy
	print dp_ave
	print dp_sd

	return doy,dp_ave,dp_sd

#======================================
#===== read observational data using interval camera (CSV)
def rd_csv_camera(indir,site_obs,year_obs):

	if year_obs == 2014 and site_obs == 'QA-S2':
		Tab_list = ['PhotoNo','date','doy','depthA']
		obs_file = '%s/%d%s_CH_monitor_season.csv'%(indir,year_obs,site_obs)
	elif year_obs == 2017 and site_obs == 'QA-S3':
		Tab_list = ['PhotoNo','date','doy','depthA','depthB','depthC','depth_ave','depth_sd']
		obs_file = '%s/%d%s_CH_monitor_season_v4.csv'%(indir,year_obs,site_obs)
	else:
		print "no ovs data skip"
		doy,dp_ave,dp_sd = -999,-999,-999
		return doy,dp_ave,dp_sd

	df = pd.read_csv(obs_file,encoding='UTF-8')

	#=== read csv data
	doy = df[Tab_list[2]].values.tolist()
	if year_obs == 2017:
		dp_ave = df[Tab_list[6]].values.tolist()
		dp_sd = df[Tab_list[7]].values.tolist()
	else:
		dp_ave = df[Tab_list[3]].values.tolist()
		dp_sd = -999
	
	print "### %s (%d) ###"%(site_obs,year_obs)

	return doy,dp_ave,dp_sd

#======================================
#===== set path & read model data
def rd_csv_mod(indir,fileN):

	Header = ['days_for_graph','month','day','hour','sd_obs'        ,'sd_calc','nsr_ice','nlr_ice','sh_ice','lh_ice',
	'melt_ice','nsr_cryo','nlr_cryo','sh_cryo','lh_cryo','melt_cryo','cryo_angle','solar_angle',
	'd_nsr','d_nlr','d_sh','d_lh','d_melt','cryoconite_depth','sr'  ,'sr_direct','sr_diffuse','src_direct','src_diffuse',
	't_direct','t_diffuse','k_direct','k_diffuse']

	Unit =   [''              ,''     ,''    ,''   ,'m (minus t=0)' ,'m'      ,'W/m2'   ,'W/m2'   ,'W/m2'  ,'W/m2',
	'W/m2'    ,'W/m2'    ,'W/m2'    ,'W/m2'   ,'W/m2'   ,'W/m2'     ,'(rad2deg)'  ,'(rad2deg)'        ,
	'W/m2' ,'W/m2' ,'W/m2','W/m2','W/m2'  ,'mm'              ,'W/m2','W/m2'  ,'W/m2'  ,'W/m2'   ,'W/m2',
	''        ,''          ,'m-1'    ,'m-1']



	if len(fileN) == 0:
		infile = '%s/ch_hourly.csv'%(indir)
	else:
		infile = '%s/ch_hourly_%s.csv'%(indir,fileN)
	print 'infile',infile

	if os.path.exists(infile):
		df = pd.read_csv(infile,encoding='UTF-8')
	else:
		print "%s does not exist"%infile 
		data = -999,-999
		return data

	#== read data size
	n = len(df[Header[0]].values.tolist())

	#=== read csv data
	data = np.zeros((len(Header),n))
	print data.shape

	for i in range(len(Header)):
		print Header[i]
		data[i,:] = np.array(df[Header[i]].values.tolist())
		print "mean, min, max = %.02f, %.02f, %.02f"%(np.mean(data[i,:]),np.min(data[i,:]),np.max(data[i,:]))

	doy = data[0,:]
	idx = Header.index('cryoconite_depth')
	ch_depth = data[idx,:]

	return doy,ch_depth

#======================================
#===== cal correlation
def cal_corr(doy_c,dpA_c,doy_m,dpA_m):
	data_corr = []

	for n in range(len(doy_c)):	
		#print doy_c[n]
		idx = np.abs(np.asarray(doy_m) - doy_c[n]).argmin()
		#print doy_m[idx]

		data_corr = np.append(data_corr,dpA_m[idx])

	bad = ~np.logical_or(np.isnan(data_corr),np.isnan(dpA_c))	# remove nan in both data
	d1 = np.compress(bad,data_corr)
	d_ref = np.compress(bad,dpA_c)

	R,P = scipy.stats.pearsonr(d1,d_ref)
	R2 = R**2
	print 'R, P, R^2 (vs. obs)= ',R,P,R2

	return R,P,R2

#======================================
#===== cal root mean square error
def cal_rmse(doy_c,dpA_c,doy_m,dpA_m):
	data_rmse = []

	for n in range(len(doy_c)):	
		#print doy_c[n]
		idx = np.abs(np.asarray(doy_m) - doy_c[n]).argmin()
		#print doy_m[idx]

		data_rmse = np.append(data_rmse,dpA_m[idx])

	bad = ~np.logical_or(np.isnan(data_rmse),np.isnan(dpA_c))	# remove nan in both data
	d1 = np.compress(bad,data_rmse)
	d_ref = np.compress(bad,dpA_c)

	RMSE = np.sqrt(mean_squared_error(d1,d_ref))
	print 'RMSE (vs. obs)=',RMSE
	
	return RMSE

#======================================
#===== cal mean absolute error
def cal_mae(doy_c,dpA_c,doy_m,dpA_m):
	data_mae = []

	for n in range(len(doy_c)):	
		#print doy_c[n]
		idx = np.abs(np.asarray(doy_m) - doy_c[n]).argmin()
		#print doy_m[idx]

		data_mae = np.append(data_mae,dpA_m[idx])

	bad = ~np.logical_or(np.isnan(data_mae),np.isnan(dpA_c))	# remove nan in both data
	d1 = np.compress(bad,data_mae)
	d_ref = np.compress(bad,dpA_c)

	MAE = mean_absolute_error(d1,d_ref)
	print 'MAE (vs. obs)=',MAE
	
	return MAE

#======================================
#===== plot figure
def plot(outdir,doy_r,dp_r_ave,dp_r_sd,doy_c,dp_c_ave,dp_c_sd,doy_m,dp_m):

	label1 = 'Obs (ruler)'
	label2 = 'Obs (camera)'
	label3 = 'Model'

	#=== set plot param
	fig, axes = plt.subplots(1, 1, figsize=(8,7))
	plt.subplots_adjust(wspace=0.15,hspace=0.60) # adjust plot space
	plt.tick_params(labelsize=18)

	#=== plot model result
	y_min, y_max = 0, 0.25

	#== set axis
	axes.set_xlim(xmin=doy_min)
	axes.set_xlim(xmax=doy_max)
	xlabels = np.arange(doy_min,doy_max+1,10)
	axes.set_xticks(xlabels)
	axes.set_xlabel('Day of year',fontsize=24)
	axes.set_ylabel('Depth of cryoconite hole (m)',fontsize=24)

	axes.set_ylim(ymin=y_min)
	axes.set_ylim(ymax=y_max)

	#== plot result (model)
	dp_m = dp_m * 0.001	# mm -> m
	axes.plot(doy_m,dp_m,color='black',linestyle='-',lw=2,label=label3)

	#== plot result (ruler)  cm -> m
	if Year in [2014,2017]:
		dp_r_ave = np.array(dp_r_ave) * 0.01
		dp_r_sd = np.array(dp_r_sd) * 0.01
		axes.errorbar(doy_r,dp_r_ave,yerr=[dp_r_sd,dp_r_sd],
			color="grey",ls='',lw=1.5,markersize=10,marker='o',ecolor='grey',capsize=2,label=label1)

	#== plot result (camera) cm -> m
	if Year in [2014,2017]:
		dp_c_ave = np.array(dp_c_ave) * 0.01


	if Year == 2014 and Site == 'QA-S2':
		axes.plot(doy_c,dp_c_ave,color='grey',linestyle='-',lw=2,label=label2)
		R,P,R2 = cal_corr(doy_c,dp_c_ave,doy_m,dp_m)
		RMSE = cal_rmse(doy_c,dp_c_ave,doy_m,dp_m)
		MAE = cal_mae(doy_c,dp_c_ave,doy_m,dp_m)

	axes.invert_yaxis()

	print fileN_m
	if Year == 2014 and Site == 'QA-S2':
		axes.text(182,0.025,'Site 2',color='black',size=24)
		axes.text(201,0.145,'$\mathregular{R^{2}}$ = %0.2f'%R2,color='black',size=20)
		axes.text(201,0.160,'RMSE = %1.2f m'%RMSE,color='black',size=20)
		axes.text(201,0.175,'MAE = %1.2f m'%MAE,color='black',size=20)
		axes.text(182,0.055,'$\Phi$ = 0.05 m',size=20)
		axes.text(182,0.073,'$D_{0}$ = 0.19 m',size=20)
		axes.text(182,0.091,r'$\alpha_{i}$ = 0.62',size=20)
		axes.text(182,0.109,'$\Delta$$T_{a}$ = +3.92 K',size=20)

	handles, labels = axes.get_legend_handles_labels()
	axes.legend(handles[::-1],labels[::-1],loc='lower right',fontsize=20)	# invert legend order

	figName = 'CHdepth_%s_%s'%(Site,fileN_m)

	if figSave == 'on':
		if os.path.isdir('%s'%outdir) == False:
			os.makedirs('%s'%outdir)	
		plt.savefig('%s/%s.png'%(outdir,figName),bbox_inches="tight")
		print 'save %s/%s.png'%(outdir,figName)
	else:
		print '%s/%s.png'%(outdir,figName)
		plt.show()
	plt.close()
#======================================

if __name__=='__main__':
    main()	
	
__author__ = 'Yukihiko Onuma <onuma@iis.u-tokyo.ac.jp>'
__version__ = '1.2.0'
__date__ = 'May. 2023'
